import Log from './Log.js'
import LogFmt from "./LogFmt.js";

export default class RedisCache {

  static create(key, client) {
    return new RedisCache(key, client)
  }

  constructor(cacheKey, client) {
    this.key = cacheKey;
    this.client = client;

    this._seconds = null;

    this.totalRequests = 0;
    this.totalHits = 0;
    this.totalMisses = 0;
  }

  async get(key) {
    this.totalRequests++;
    const namespacedKey = `${this.key}:${key}`;
    const value = await this.client.get(namespacedKey);
    if (value) {
      this.totalHits++;
      Log.debug(`Using '${this.key}' cache`);
      return JSON.parse(value);
    } else {
      this.totalMisses++;
      return null;
    }
  }

  async add(key, value) {
    const namespacedKey = `${this.key}:${key}`;
    const stringValue = JSON.stringify(value);
    if (this._seconds) {
      await this.client.set(namespacedKey, stringValue, { EX: this._seconds });
    } else {
      await this.client.set(namespacedKey, stringValue);
    }
  }

  async delete(key) {
    const namespacedKey = `${this.key}:${key}`;
    await this.client.del(namespacedKey);
  }

  async clear() {
    const pattern = `${this.key}:*`;

    for await (const key of this.client.scanIterator({ MATCH: pattern, COUNT: 100 })) {
      console.log(key);
      await this.client.del(key);
    }
  }

  hours(hours) {
    return this.minutes(hours * 60);
  }

  minutes(minutes) {
    return this.seconds(minutes * 60)
  }

  seconds(seconds) {
    return this._clearSeconds(seconds);
  }

  milliseconds(milliseconds) {
    const seconds = Math.round(milliseconds / 1000);
    return this._clearSeconds(seconds);
  }

  logInit() {
    const logFmt = LogFmt.msg('Init cache')
      .tag('cache').string('cache-name', this.key);
    if (this._seconds) {
      logFmt.number('cache-seconds', this._seconds);
    }

    Log.info(logFmt.log());
  }

  _clearSeconds(seconds) {
    this._seconds = seconds;
    return this;
  }

}
